@@ -6,17 +6,19 @@ import inspect
 import re
 import copy
 import itertools
+import numbers
 
 
 # Python3 support
 py3 = sys.version_info[0] == 3
 if py3:
     import urllib.parse as urlparse
-    basestring = (str, bytes)
+    str_types = (str, bytes)
     unicode = str
 else:
     from future_builtins import map
     import urlparse
+    str_types = (str, unicode)
 
 
 """
@@ -133,7 +135,7 @@ class Trafaret(object):
         Helper for complex trafarets, takes trafaret instance or class
         and returns trafaret instance
         """
-        if isinstance(trafaret, Trafaret):
+        if isinstance(trafaret, Trafaret) or inspect.isroutine(trafaret):
             return trafaret
         elif issubclass(trafaret, Trafaret):
             return trafaret()
@@ -353,8 +355,10 @@ class Float(Trafaret):
     <Float(gte=1, lte=10)>
     >>> Float().check(1.0)
     1.0
-    >>> extract_error(Float(), 1)
+    >>> extract_error(Float(), 1 + 3j)
     'value is not float'
+    >>> extract_error(Float(), 1)
+    1.0
     >>> Float(gte=2).check(3.0)
     3.0
     >>> extract_error(Float(gte=2), 1.0)
@@ -370,6 +374,7 @@ class Float(Trafaret):
     __metaclass__ = NumberMeta
 
     value_type = float
+    convertable = str_types + (numbers.Real,)
 
     def __init__(self, gte=None, lte=None, gt=None, lt=None):
         self.gte = gte
@@ -379,7 +384,7 @@ class Float(Trafaret):
 
     def _check_val(self, val):
         if not isinstance(val, self.value_type):
-            if isinstance(val, basestring):
+            if isinstance(val, self.convertable):
                 try:
                     value = self.value_type(val)
                 except ValueError:
@@ -425,6 +430,8 @@ class Int(Float):
     >>> Int().check(5)
     5
     >>> extract_error(Int(), 1.1)
+    1
+    >>> extract_error(Int(), 1 + 1j)
     'value is not int'
     """
 
@@ -471,10 +478,10 @@ class String(Trafaret):
 
     def __init__(self, allow_blank=False, regex=None):
         self.allow_blank = allow_blank
-        self.regex = re.compile(regex) if isinstance(regex, basestring) else regex
+        self.regex = re.compile(regex) if isinstance(regex, str_types) else regex
 
     def _check_val(self, value):
-        if not isinstance(value, basestring):
+        if not isinstance(value, str_types):
             self._failure("value is not string")
         if not self.allow_blank and len(value) is 0:
             self._failure("blank value is not allowed")
@@ -486,7 +493,7 @@ class String(Trafaret):
         return value
 
     def converter(self, value):
-        if isinstance(value, basestring):
+        if isinstance(value, str_types):
             return value
         return value.group()
 
@@ -627,7 +634,7 @@ class List(Trafaret):
     [1, 2, 3]
     >>> List(String).check(["foo", "bar", "spam"])
     ['foo', 'bar', 'spam']
-    >>> extract_error(List(Int), [1, 2, 3.0])
+    >>> extract_error(List(Int), [1, 2, 1 + 3j])
     {2: 'value is not int'}
     >>> List(Int, min_length=1).check([1, 2, 3])
     [1, 2, 3]
@@ -695,15 +702,13 @@ class Key(object):
         self.trafaret = Any()
 
     def pop(self, data):
-        if self.name not in data:
-            if self.optional:
-                raise StopIteration
-            elif self.default is not None:
-                pass
-            else:
-                yield self.name, DataError(error='is required')
-                raise StopIteration
-        yield self.get_name(), data.pop(self.name, self.default)
+        if self.name in data or self.default is not None:
+            yield self.get_name(), catch_error(self.trafaret,
+                    data.pop(self.name, self.default))
+            raise StopIteration
+        if not self.optional:
+            yield self.name, DataError(error='is required')
+        raise StopIteration
 
     def set_trafaret(self, trafaret):
         self.trafaret = trafaret
@@ -789,10 +794,7 @@ class Dict(Trafaret):
                 if isinstance(v, DataError):
                     errors[k] = v
                 else:
-                    try:
-                        collect[k] = key.trafaret.check(v)
-                    except DataError as err:
-                        errors[k] = err
+                    collect[k] = v
         for key in data:
             if not self.allow_any and key not in self.extras:
                 errors[key] = DataError("%s is not allowed key" % key)
@@ -827,6 +829,7 @@ class Mapping(Trafaret):
     >>> trafaret
     <Mapping(<String> => <Int>)>
     >>> trafaret.check({"foo": 1, "bar": 2})
+    {'foo': 1, 'bar': 2}
     >>> extract_error(trafaret, {"foo": 1, "bar": None})
     {'bar': {'value': 'value is not int'}}
     >>> extract_error(trafaret, {"foo": 1, 2: "bar"})
@@ -856,6 +859,7 @@ class Mapping(Trafaret):
                 checked_mapping[checked_key] = checked_value
         if errors:
             raise DataError(error=errors)
+        return checked_mapping
 
     def __repr__(self):
         return "<Mapping(%r => %r)>" % (self.key, self.value)
@@ -905,7 +909,8 @@ class Call(Trafaret):
     """
     >>> def validator(value):
     ...     if value != "foo":
-    ...         return "I want only foo!"
+    ...         return DataError("I want only foo!")
+    ...     return 'foo'
     ...
     >>> trafaret = Call(validator)
     >>> trafaret
@@ -925,10 +930,12 @@ class Call(Trafaret):
                                " one argument function")
         self.fn = fn
 
-    def _check(self, value):
-        error = self.fn(value)
-        if error is not None:
-            self._failure(error)
+    def _check_val(self, value):
+        res = self.fn(value)
+        if isinstance(res, DataError):
+            raise res
+        else:
+            return res
 
     def __repr__(self):
         return "<CallC(%s)>" % self.fn.__name__
@@ -941,14 +948,14 @@ class Forward(Trafaret):
     >>> node << Dict(name=String, children=List[node])
     >>> node
     <Forward(<Dict(children=<List(<recur>)>, name=<String>)>)>
-    >>> node.check({"name": "foo", "children": []})
-    {'children': [], 'name': 'foo'}
+    >>> node.check({"name": "foo", "children": []}) == {'children': [], 'name': 'foo'}
+    True
     >>> extract_error(node, {"name": "foo", "children": [1]})
     {'children': {0: 'value is not dict'}}
     >>> node.check({"name": "foo", "children": [ \
                         {"name": "bar", "children": []} \
-                     ]})
-    {'children': [{'children': [], 'name': 'bar'}], 'name': 'foo'}
+                     ]}) == {'children': [{'children': [], 'name': 'bar'}], 'name': 'foo'}
+    True
     """
 
     def __init__(self):
@@ -1045,10 +1052,10 @@ def guard(trafaret=None, **kwargs):
                                          argspec.defaults or ()):
                     if name not in call_args:
                         call_args[name] = default
-                trafaret.check(call_args)
+                converted = trafaret.check(call_args)
             except DataError as err:
                 raise GuardError(error=err.error)
-            return fn(*args, **kwargs)
+            return fn(**converted)
         decor.__doc__ = "guarded with %r\n\n" % trafaret + (decor.__doc__ or "")
         return decor
     return wrapper
@@ -1067,7 +1074,7 @@ def ignore(val):
     pass
 
 
-def extract_error(checker, *a, **kw):
+def catch_error(checker, *a, **kw):
 
     """
     Helper for tests - catch error and return it as dict
@@ -1079,4 +1086,15 @@ def extract_error(checker, *a, **kw):
         elif callable(checker):
             return checker(*a, **kw)
     except DataError as error:
-        return error.as_dict()
+        return error
+
+def extract_error(checker, *a, **kw):
+
+    """
+    Helper for tests - catch error and return it as dict
+    """
+
+    res = catch_error(checker, *a, **kw)
+    if isinstance(res, DataError):
+        return res.as_dict()
+    return res
